#! /bin/bash
# InnoTek VirtualBox
# Linux kernel module init script
#
# Copyright (C) 2006 InnoTek Systemberatung GmbH
#
# This file is part of VirtualBox Open Source Edition (OSE), as
# available from http://www.virtualbox.org. This file is free software;
# you can redistribute it and/or modify it under the terms of the GNU
# General Public License as published by the Free Software Foundation,
# in version 2 as it comes in the "COPYING" file of the VirtualBox OSE
# distribution. VirtualBox OSE is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY of any kind.
#
# If you received this file as part of a commercial VirtualBox
# distribution, then only the terms of your commercial VirtualBox
# license agreement apply instead of the previous paragraph.


# chkconfig: 35 30 60
# description: VirtualBox Linux kernel module
#
### BEGIN INIT INFO
# Provides:       vboxdrv
# Required-Start: $syslog
# Required-Stop:
# Default-Start:  3 5
# Default-Stop:
# Description:    VirtualBox Linux kernel module
### END INIT INFO

system=unknown
if [ -f /etc/redhat-release ]; then
    system=redhat
elif [ -f /etc/SuSE-release ]; then
    system=suse
elif [ -f /etc/debian_version ]; then
    system=debian
elif [ -f /etc/gentoo-release ]; then
    system=gentoo
else
    echo "$0: Unknown system" 1>&2
fi

if [ $system = redhat ]; then
    . /etc/init.d/functions
    fail_msg() {
        echo_failure
        echo
    }

    succ_msg() {
        echo_success
        echo
    }

    begin() {
        echo -n $1
    }
fi

if [ $system = suse ]; then
    . /etc/rc.status
    fail_msg() {
        rc_failed 1
        rc_status -v
    }

    succ_msg() {
        rc_reset
        rc_status -v
    }

    begin() {
        echo -n $1
    }
fi

if [ $system = debian ]; then
    fail_msg() {
        echo "...fail!"
    }

    succ_msg() {
        echo "...done."
    }

    begin() {
        echo -n $1
    }
fi

if [ $system = gentoo ]; then
    . /sbin/functions.sh
    fail_msg() {
        eend 1
    }

    succ_msg() {
        eend $?
    }

    begin() {
        ebegin $1
    }

    if [ "`which $0`" = "/sbin/rc" ]; then
        shift
    fi
fi

kdir=/lib/modules/`uname -r`/kernel/3rdparty/vbox
dev=/dev/vboxdrv
modname=vboxdrv
module=$kdir/$modname
groupname=vboxusers

file=""
test -f $module.o  && file=$module.o
test -f $module.ko.gz && file=$module.ko.gz

fail() {
    if [ $system = gentoo ]; then
        eerror $1
        exit 1
    fi
    echo -n "($1)"
    fail_msg
    exit 1
}

test -z "$file" && {
    fail "Kernel module not found"
}

running() {
    lsmod | grep -q $modname[^_-]
}

start() {
    begin "Starting VirtualBox kernel module ";
    running || {
        rm -f $dev || {
            fail "Cannot remove $dev"
        }

        modprobe $modname || {
            fail "modprobe $modname failed"
        }

        sleep 1
    }
    if [ ! -c $dev ]; then
        maj=`sed -n 's;\([0-9]\+\) vboxdrv;\1;p' /proc/devices`
        test -z $maj && {
            rmmod $modname
            fail "Cannot locate device major"
        }

        mknod -m 0666 $dev c $maj 0 || {
            rmmod $modname
            fail "Cannot create device $dev with major $maj"
        }
    fi

    chown :$groupname $dev || {
        rmmod $modname
        fail "Cannot change owner $groupname for device $dev"
    }

# the next line gives all users permssion to use virtualbox
# to use groups permisson change chmod to 664

   chmod -c 666 $dev || {
        rmmod $modname
        fail "Cannot change permissions for device $dev"
    }

    succ_msg
    return 0
}

stop() {
    begin "Stopping VirtualBox kernel module ";
    if running; then
        rmmod $modname || fail "Cannot unload module $modname"
        rm -f $dev || fail "Cannot unlink $dev"
    fi
    succ_msg
    return 0
}

restart() {
    stop && start
    return 0
}

dmnstatus() {
    if running; then
        echo "VirtualBox kernel module is loaded."
    else
        echo "VirtualBox kernel module is not loaded."
    fi
}

case "$1" in
start)
    start
    ;;
stop)
    stop
    ;;
restart)
    restart
    ;;
status)
    dmnstatus
    ;;
*)
    echo "Usage: $0 {start|stop|restart|status}"
    exit 1
esac

exit
